import { Component, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from '../../core/services/auth/auth.service';
import { UserDialogComponent } from '../misc/navigation/user-dialog/user-dialog.component';
import { Principal } from '../../core/model/auth/Principal';
import { AppRole } from '../../core/common/enum/app-role';

declare interface RouteInfo {
	path: string;
	title: string;
	icon: string;
}
declare interface GroupMenuItem {
	title: string;
	routes: RouteInfo[];
	requiresAuthentication: boolean;
	requiresAdmin: boolean;
}

export const GENERAL_ROUTES: RouteInfo[] = [
	{ path: '/home', title: 'SIDE-BAR.DASHBOARD', icon: 'dashboard' }
];
export const DMP_ROUTES: RouteInfo[] = [
	{ path: '/plans', title: 'SIDE-BAR.MY-DMPS', icon: 'view_agenda' },
	{ path: '/datasets', title: 'SIDE-BAR.MY-DATASET-DESC', icon: 'library_books' },
	{ path: '/projects', title: 'SIDE-BAR.MY-PROJECTS', icon: 'work_outline' }
];
export const ADMIN_ROUTES: RouteInfo[] = [
	{ path: '/dmp-profiles', title: 'SIDE-BAR.DMP-TEMPLATES', icon: 'library_books' },
	{ path: '/dataset-profiles', title: 'SIDE-BAR.DATASET-TEMPLATES', icon: 'library_books' },
	{ path: '/users', title: 'SIDE-BAR.USERS', icon: 'people' }
];
// export const HISTORY_ROUTES: RouteInfo[] = [
//   { path: '/typography', title: 'SIDE-BAR.HISTORY-VISITED',  icon: 'visibility'},
//   { path: '/icons', title: 'SIDE-BAR.HISTORY-EDITED',  icon: 'edit'}
// ];
export const PUBLIC_ROUTES: RouteInfo[] = [
	{ path: '/explore', title: 'SIDE-BAR.PUBLIC-DESC', icon: 'public', },
	{ path: '/explore-plans', title: 'SIDE-BAR.PUBLIC-DMPS', icon: 'public', }
];

@Component({
	selector: 'app-sidebar',
	templateUrl: './sidebar.component.html',
	styleUrls: ['./sidebar.component.css', './sidebar.component.scss']
})
export class SidebarComponent implements OnInit {
	generalItems: GroupMenuItem;
	dmpItems: GroupMenuItem;
	adminItems: GroupMenuItem;
	//   historyItems: GroupMenuItem;
	publicItems: GroupMenuItem;
	groupMenuItems: GroupMenuItem[] = [];

	constructor(public translate: TranslateService, private authentication: AuthService, private dialog: MatDialog) { }

	ngOnInit() {
		this.generalItems = {
			title: 'SIDE-BAR.GENERAL',
			routes: GENERAL_ROUTES,
			requiresAuthentication: false,
			requiresAdmin: false,
		}
		this.groupMenuItems.push(this.generalItems);

		this.dmpItems = {
			title: 'SIDE-BAR.DMP',
			routes: DMP_ROUTES,
			requiresAuthentication: true,
			requiresAdmin: false
		}
		this.groupMenuItems.push(this.dmpItems);

		this.adminItems = {
			title: 'SIDE-BAR.ADMIN',
			routes: ADMIN_ROUTES,
			requiresAuthentication: true,
			requiresAdmin: true
		}
		this.groupMenuItems.push(this.adminItems);

		// this.historyItems = {
		// 	title: 'SIDE-BAR.HISTORY',
		// 	routes: HISTORY_ROUTES
		// }
		// this.groupMenuItems.push(this.historyItems);

		this.publicItems = {
			title: 'SIDE-BAR.PUBLIC',
			routes: PUBLIC_ROUTES,
			requiresAuthentication: false,
			requiresAdmin: false
		}
		this.groupMenuItems.push(this.publicItems);
	}

	public principalHasAvatar(): boolean {
		return this.authentication.current().avatarUrl != null;
	}

	public getPrincipalAvatar(): string {
		return this.authentication.current().avatarUrl;
	}

	public isAuthenticated(): boolean {
		const myBollean = this.isAdmin();
		return !(!this.authentication.current());
	}

	public isAdmin(): boolean {
		const principal: Principal = this.authentication.current();
		if (principal) {
			if (principal.authorities.find(role => role === AppRole.Admin)) {
				return true;
			}
			else {
				return false;
			}
		} else {
			return false;
		}
	}

	showItem(value: GroupMenuItem) {
		if (value.requiresAdmin) {
			return this.isAdmin();
		}
		else {
			return this.isAuthenticated() || !value.requiresAuthentication;
		}
	}

	openProfile() {
		const dialogRef = this.dialog.open(UserDialogComponent, {
			hasBackdrop: true,
			autoFocus: false,
			closeOnNavigation: true,
			disableClose: false,
			position: { top: '64px', right: '1em' }
		});
	}
}
